<?php

namespace Drupal\data_pipelines\Commands;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\data_pipelines\Entity\DatasetInterface;
use Drupal\data_pipelines\Form\DatasetBatchOperations;
use Drush\Commands\DrushCommands;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Data pipelines drush commands.
 */
class DataPipelinesCommands extends DrushCommands {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Creates a DataPipelinesCommand object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct();
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Provide a list of datasets if argument is not provided.
   *
   * @hook interact data-pipelines:reindex
   */
  public function datasetReindexInteract(InputInterface $input, OutputInterface $output) {
    if (!empty($input->getArgument('dataset_id'))) {
      return;
    }

    $datasets = $this->entityTypeManager->getStorage('data_pipelines')->loadMultiple();
    $choices = [];
    foreach ($datasets as $dataset) {
      // Choices does not support integer keys, so do some string manipulation.
      $choices[(string) 'id_' . $dataset->id()] = $dataset->label();
    }
    $choice = $this->io()->choice(dt('Which dataset would you like to reindex'), $choices);

    $input->setArgument('dataset_id', (int) substr($choice, 3));
  }

  /**
   * Rebulid the index for a dataset.
   *
   * @param int $dataset_id
   *   The data pipelines entity id to rebuild.
   *
   * @command data-pipelines:reindex
   *
   * @validate-module-enabled data_pipelines
   */
  public function datasetReindex(int $dataset_id) {
    $dataset = $this->entityTypeManager->getStorage('data_pipelines')->load($dataset_id);

    assert($dataset instanceof DatasetInterface);
    batch_set(DatasetBatchOperations::batchForDataset($dataset));
    drush_backend_batch_process();
  }

}
